{

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
    *                                                                 *
    *   Copyrightcillas 1.0.3                                         *
    *   Copyright  2004  Florian Oldach <oldach@hartwork.org>        *
    *                     Sebastian Pipping <pipping@hartwork.org>    *
    *                                                                 *
    *   --> http://www.hartwork.org                                   *
    *                                                                 *
    *                                                                 *
    *   This source code is released under LGPL.                      *
    *   See LGPL.txt for details.                        2004-09-01   *
    *                                                                 *
    \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

}

PROGRAM crc;

USES
  crt,
  crc_std,
  crc_intr,
  crc_menu,
  crc_scor,
  crc_set,
  crc_worm;

TYPE
  pSnakeList = ^tSnakeList;
  tSnakeList = RECORD
    wSnake : tWorm;
    pNext  : pSnakeList;

  END;

  pRankList = ^tRankList;
  tRankList = RECORD
    iColor       : BYTE;
    iPoints      : LONGINT;
    iBonus       : BYTE;
    iFinalPoints : LONGINT;
    pNext        : pRankList;

  END;

VAR
  g_iParamIndex     : BYTE;
  g_bSkipIntro      : BOOLEAN;

  g_iSnakeColor     : ARRAY[ 1..MAX_PLAYERS ] OF BYTE;
  g_sSnakeColorName : ARRAY[ 1..MAX_PLAYERS ] OF STRING[ 5 ];

  g_fMap            : tFileMap;
  g_iMaps           : BYTE;

  g_bQuit           : BOOLEAN;


{ ---------------------------------------------------------------------------------------------------- add_cookie ----- }
FUNCTION add_cookie( VAR map : tMap ) : BOOLEAN; { Adds a cookie at a random position }
  CONST
    RAND_MAX = MAP_WIDTH * MAP_HEIGHT;

  TYPE
    pRep = ^tRep;
    tRep = RECORD
      iOld  : WORD;
      iRep  : WORD;
      pNext : pRep;

    END;

  VAR
    iRoundRand       : WORD;
    iPos, iPosOrg    : WORD;
    x, y             : BYTE;
    RepFirst, RepCur : pRep;
    bDone, bRepFound : BOOLEAN;

  BEGIN
    {
    We list all positions we checked already.
    This way we can reduce the times RANDOM is called
    which should speed up the whole thing especially on
    a more populated map.

    The mechanism used ensures that each number will not
    occur more than one time.

    EXAMPLE:
    We want to generate unique numbers from 0 to 15.

    (1) We call RANDOM( 16 ) and get... [4].
        Replacement list is empty so [4] is our first number.
        Now we add [4->15] to the replacement list so we'll
        get [15] when [4] occurs next time.

    (2) We call RANDOM( 15 ) and get... [7].
        We search the replacement list but do not find a
        matching entry so [7] is our second number.
        Now we add [7->14] to the replacement list.

    (3) We call RANDOM( 14 ) and get... [4].
        This time we find a matching entry in the
        replacement list (4->15).
        We replace this entry with [4->13] and our
        third number is [15].

    (x) ...

    The algorith used below uses a single-linked list.
    Using a binary tree instead would speed up searching but
    also make cleanup recursive or very complicated which
    we decided to avoid here.
    }

    bDone    := FALSE;
    RepFirst := NIL;

    iRoundRand := RAND_MAX;
    REPEAT
      {
      We cannot generate [x] and [y] separatly if
      we want all points to have the same probabilty.
      }

      iPos := RANDOM( iRoundRand );
      DEC( iRoundRand );

      { Search for replacement }
      RepCur := RepFirst;
      bRepFound := FALSE;
      WHILE NOT ( ( RepCur = NIL ) OR bRepFound ) DO
        BEGIN
          IF ( RepCur^.iOld = iPos ) THEN
            BEGIN
              { Found }
              iPosOrg := iPos;        { Remember original index }
              iPos := RepCur^.iRep;   { Replace index }
              bRepFound := TRUE;      { Stop searching }

            END
          ELSE
            RepCur := RepCur^.pNext;

        END;


      { Index to point }
      x := ( iPos MOD MAP_WIDTH ) + 1;
      y := ( iPos DIV MAP_WIDTH ) + 1;


      { Can we add a cookie here? / check for space }
      IF ( map[ x, y ] = ASCII_SPACE ) THEN
        BEGIN
          { Add cookie }
          map[ x, y ] := ASCII_COOKIE;
          GOTOXY( x, y );
          WRITE( ASCII_COOKIE );
          bDone := TRUE;

        END
      ELSE
        BEGIN
          { Add replacement }
          IF bRepFound THEN
            BEGIN
              { Replace entry }
              IF ( iRoundRand <> iPosOrg ) THEN
                RepCur^.iRep := iRoundRand

            END
          ELSE
            BEGIN
              { Add entry at beginning }
              new( RepCur );
              RepCur^.iOld  := iPos;
              RepCur^.iRep  := iRoundRand;
              RepCur^.pNext := RepFirst;

              RepFirst := RepCur;

            END;

        END;

    UNTIL bDone;


    { Delete replacement list }
    RepCur := RepFirst;
    WHILE ( RepCur <> NIL ) DO
      BEGIN
        RepCur := RepCur^.pNext;
        DISPOSE( RepFirst );
        RepFirst := RepCur;

      END;


    add_cookie := bDone;

  END;


{ ---------------------------------------------------------------------------------------------------- status_bar ----- }
PROCEDURE status_bar( MySnakes : pSnakeList ); { Draw bar showing the scores }
  VAR
    iSnakeIndex : BYTE;

  BEGIN
    GOTOXY( 1, 50 );
    iSnakeIndex := 1;
    WHILE ( MySnakes <> NIL ) DO
      BEGIN
        TEXTCOLOR( g_iSnakeColor[ iSnakeIndex ] );
        WRITE( 'Points: ', zero( MySnakes^.wSnake.iPoints, 6 ), '   ' );

        MySnakes := MySnakes^.pNext;
        INC( iSnakeIndex );

      END;
    TEXTCOLOR( LIGHTGRAY );

  END;


{ ------------------------------------------------------------------------------------------------------ cut_last ----- }
PROCEDURE cut_last( VAR MyName : tName ); { Cuts off last charactor of a name }
  VAR
    sTemp : tName;
    iLen  : BYTE;
    i     : BYTE;

  BEGIN
    sTemp := '';
    iLen := LENGTH( MyName ) - 1;

    { Build new name }
    FOR i := 1 TO iLen DO
      sTemp := sTemp + MyName[ i ];
    MyName := sTemp;

  END;


{ ---------------------------------------------------------------------------------------------- game_over_single ----- }
PROCEDURE game_over_single( MyPoints : WORD );
  VAR
    sName    : tName;
    iCurLen  : BYTE;
    chKey    : CHAR;
    bQuit    : BOOLEAN;
    bSyskey  : BOOLEAN;

  BEGIN
    { Will (s)he join the highscores? }
    IF will_join( MyPoints ) THEN
      BEGIN
        frame_center( 30, 8, FALSE );
        WINDOW_CENTER( 28, 6, 0, 0);
        CLRSCR;
        WINDOW_CENTER( 24, 6, 0, 0 );

        GOTOXY( 1, 2 );
        WRITELN_CENTER( 'New highscore!' );
        WRITELN;
        WRITELN( 'Please enter your name:' );
        WRITE( ASCII_UNDERLINE );

        sName   := '';
        iCurLen := 0;
        bQuit   := FALSE;
        bSyskey := FALSE;

        empty_buffer;
        REPEAT
          chKey := READKEY;
          IF bSyskey THEN
            bSyskey := FALSE
          ELSE
            BEGIN
              CASE chKey OF

                KEY_NULL:
                  bSyskey := TRUE;

                KEY_RETURN:
                  BEGIN
                    IF ( iCurLen > 0 ) THEN
                      bQuit := TRUE;

                  END;

                KEY_ESCAPE:
                  BEGIN
                    sName := '';
                    bQuit := TRUE;

                  END;

                KEY_BACKSPACE:
                  BEGIN
                    IF ( iCurLen > 0 ) THEN
                      BEGIN
                        GOTOXY( iCurLen, WHEREY );
                        WRITE( ASCII_UNDERLINE, ASCII_SPACE );

                        cut_last( sName );
                        DEC( iCurLen );

                      END;

                  END

              ELSE
                IF ( iCurLen < MAX_NAME_LENGTH ) THEN
                  BEGIN
                    INC( iCurLen );
                    sName := sName + chKey;
                    GOTOXY( iCurLen, WHEREY );
                    WRITE( chKey, ASCII_UNDERLINE );

                  END;

              END; { CASE }

            END; { ELSE }

        UNTIL bQuit;

        IF ( sName <> '' ) THEN
          update_scores( sName, MyPoints );

      END
    ELSE
      BEGIN
        frame_center( 22, 5, FALSE );
        WINDOW_CENTER( 20, 3, 0, 0 );
        CLRSCR;
        GOTOXY( 1, 2 );
        WRITELN_CENTER( 'G-A-M-E  O-V-E-R' );
        special_key;

      END;

  END;


{ ---------------------------------------------------------------------------------------------------- rank_names ----- }
PROCEDURE rank_names( pRank : pRankList );
  VAR
   iRank        : BYTE;
   iRankPoints  : WORD;
   iSnakeIndex  : BYTE;

  BEGIN
    iSnakeIndex := 0;
    iRank := 0;
    iRankPoints := 65535;

    WHILE( pRank <> NIL )DO
      BEGIN
        INC( iSnakeIndex );

        IF ( pRank^.iFinalPoints < iRankPoints ) THEN
          BEGIN
            iRankPoints := pRank^.iFinalPoints;
            INC( iRank );

          END;

        GOTOXY( 1, iSnakeIndex + 2 );
        WRITE( iRank, '.' );

        TEXTCOLOR( g_iSnakeColor[ pRank^.iColor ] );
        GOTOXY( 4, iSnakeIndex + 2 );
        WRITE( g_sSnakeColorName[ pRank^.iColor ] );
        TEXTCOLOR( LIGHTGRAY );

        pRank := pRank^.pNext;

      END;

  END;


{ --------------------------------------------------------------------------------------------------- rank_points ----- }
PROCEDURE rank_points( pRank : pRankList );
  VAR
   iSnakeIndex  : BYTE;

  BEGIN
    iSnakeIndex := 0;

    WHILE( pRank <> NIL )DO
      BEGIN
        INC( iSnakeIndex );

        TEXTCOLOR( g_iSnakeColor[ pRank^.iColor ] );
        GOTOXY( 12, iSnakeIndex + 2 );
        WRITE( pRank^.iPoints:6 );
        TEXTCOLOR( LIGHTGRAY );
        WRITE( ' + ', pRank^.iBonus:3 );

        pRank := pRank^.pNext;

      END;

  END;


{ ----------------------------------------------------------------------------------------------- game_over_multi ----- }
PROCEDURE game_over_multi( VAR MySnakes : pSnakeList; iRound : BYTE; VAR bEscape : BOOLEAN );
  VAR
    T0Snakes   : pSnakeList;

    FirstRank   : pRankList;
    T0Rank      : pRankList;
    T1Rank      : pRankList;
    T2Rank      : pRankList;

    iSumBonus   : WORD;

    iSnakeIndex : BYTE;

  BEGIN
    frame_center( 29, g_Settings.iPlayers + 6, FALSE );
    window_center( 27, g_Settings.iPlayers + 4, 0, 0 );
    CLRSCR;
    window_center( 24, g_Settings.iPlayers + 3, 0, ( ( g_Settings.iPlayers AND 1 ) + 1 ) MOD 2 );

    GOTOXY( 6, 1 );
    WRITE( 'ROUND ', iRound,' OF ', g_iRounds[ g_Settings.iRoundIndex ] );

    iSumBonus := 0;
    iSnakeIndex := 1;
    T0Snakes := MySnakes;

    NEW( T2Rank );
    T2Rank^.iColor := iSnakeIndex;
    T2Rank^.iPoints := T0Snakes^.wSnake.iPoints;
    IF ( T0Snakes^.wSnake.bCrash ) THEN
      BEGIN
        T2Rank^.iBonus := 0;
        T0Snakes^.wSnake.bCrash := FALSE;

      END
    ELSE
      BEGIN
        T2Rank^.iBonus := 100;
        T0Snakes^.wSnake.iPoints := T2Rank^.iPoints + T2Rank^.iBonus;

      END;
    T2Rank^.iFinalPoints := T0Snakes^.wSnake.iPoints;
    iSumBonus := iSumBonus + T2Rank^.iBonus;

    FirstRank := T2Rank;
    T0Snakes := T0Snakes^.pNext;

    WHILE ( T0Snakes <> NIL ) DO
      BEGIN
        INC( iSnakeIndex );

        NEW( T1Rank );
        T2Rank^.pNext := T1Rank;
        T2Rank := T1Rank;

        T2Rank^.iColor := iSnakeIndex;
        T2Rank^.iPoints := T0Snakes^.wSnake.iPoints;
        IF ( T0Snakes^.wSnake.bCrash ) THEN
          BEGIN
            T2Rank^.iBonus := 0;
            T0Snakes^.wSnake.bCrash := FALSE;

          END
        ELSE
          BEGIN
            T2Rank^.iBonus := 100;
            T0Snakes^.wSnake.iPoints := T2Rank^.iPoints + T2Rank^.iBonus;

          END;
        T2Rank^.iFinalPoints := T0Snakes^.wSnake.iPoints;
        iSumBonus := iSumBonus + T2Rank^.iBonus;

        T0Snakes := T0Snakes^.pNext;

      END; { WHILE }

    T2Rank^.pNext := NIL;

    { Sort ranklist }
    T0Rank := FirstRank^.pNext;
    FirstRank^.pNext := NIL;
    WHILE( T0Rank <> NIL )DO
      BEGIN
        IF ( T0Rank^.iFinalPoints > FirstRank^.iFinalPoints ) THEN
          BEGIN
            T1Rank := T0Rank^.pNext;

            T0Rank^.pNext := FirstRank;
            FirstRank := T0Rank;

            T0Rank := T1Rank;

          END
        ELSE
          BEGIN
            T1Rank := FirstRank;
            T2Rank := T1Rank^.pNext;
            WHILE ( T0Rank^.iFinalPoints <= T2Rank^.iFinalPoints ) AND ( T2Rank <> NIL ) DO
              BEGIN
                T1Rank := T2Rank;
                T2Rank := T2Rank^.pNext;

              END;

            T1Rank^.pNext := T0Rank;
            T1Rank := T0Rank;
            T0Rank := T0Rank^.pNext;
            T1Rank^.pNext := T2Rank;

          END; { ELSE }

      END; { WHILE }

    rank_names( FirstRank );
    rank_points( FirstRank );
    IF continue THEN
      BEGIN
        IF ( iSumBonus > 0 ) THEN
          BEGIN
            FOR iSnakeIndex := 1 TO 100 DO
              BEGIN
                T1Rank := FirstRank;
                WHILE ( T1Rank <> NIL ) DO
                  BEGIN
                    IF ( T1Rank^.iBonus >= 1 ) THEN
                      BEGIN
                        DEC( T1Rank^.iBonus );
                        INC( T1Rank^.iPoints );

                      END;

                    T1Rank := T1Rank^.pNext;

                  END; { WHILE }

                DELAY( 2 );
                rank_points( FirstRank );

              END; { FOR }

            IF ( iRound = g_iRounds[ g_Settings.iRoundIndex ] ) THEN
              BEGIN
                DELAY( 2000 );
                special_key;

              END
            ELSE
              bEscape := NOT continue;

          END; { IF }

        END
      ELSE
        bEscape := TRUE;

      window_full_50;

      { Kill ranklist }
      T2Rank := T1Rank;
      WHILE ( T2Rank <> NIL ) DO
        BEGIN
          T1Rank := T2Rank^.pNext;
          DISPOSE( T2Rank );
          T2Rank := T1Rank;

        END;

  END;


{ ---------------------------------------------------------------------------------------------------- game_error ----- }
PROCEDURE game_error( sText : STRING );
  BEGIN
    GOTOXY( 1, 1 );
    WRITE( sText );
    special_key;
    res_small;

  END;


{ ---------------------------------------------------------------------------------------------------------- game ----- }
PROCEDURE game;
  VAR
    iRouteOffset   : BYTE;
    MyMap          : tMapEx;
    x, y           : BYTE;

    iCookieCounter : BYTE;

    T0Slice        : pWormSlice;

    iDelayTimes    : BYTE;
    iRound         : WORD;

    bCrash         : BOOLEAN;
    bEscape        : BOOLEAN;
    bDone          : BOOLEAN;

    iSnakeIndex    : BYTE;

    FirstSnake     : pSnakeList;
    T1Snake        : pSnakeList;
    T2Snake        : pSnakeList;

    bAllowPaint    : BOOLEAN;

    bNewScores     : BOOLEAN;

    iDelayCounter  : BYTE;
    iControlIndex  : BYTE;
    chKey          : CHAR;
    bFound         : BOOLEAN;

    iStepCounter   : LONGINT;



    bTemp          : BOOLEAN;

  BEGIN
    iRouteOffset := ( g_Settings.iPlayers - 1 ) * g_Settings.iPlayers DIV 2;
    iDelayTimes  := 13 - g_Settings.iSpeed;
    iRound       := 1;

    bEscape      := FALSE;
    bDone        := FALSE;

    res_big;
    empty_buffer;

    { Create snakelist }
    NEW( T1Snake );
    T1Snake^.pNext := NIL;
    FirstSnake := T1Snake;
    iSnakeIndex := 1;
    WHILE ( iSnakeIndex < g_Settings.iPlayers ) DO
      BEGIN
        NEW( T2Snake );
        T2Snake^.pNext := NIL;
        T1Snake^.pNext := T2Snake;
        T1Snake := T2Snake;

        INC( iSnakeIndex );

      END;

    REPEAT { GAME ROUNDS }
      bNewScores := FALSE;
      bCrash := FALSE;

      { Read map }
      {$I-}
      RESET( g_fMap );
      {$I+}

      IF ( IORESULT <> 0 ) THEN
        BEGIN
          game_error( 'Mapfile could not be opened' );
          EXIT;

        END;

      IF ( g_Settings.iMap < FILESIZE( g_fMap ) ) THEN
        SEEK( g_fMap, g_Settings.iMap );
      READ( g_fMap, MyMap );
      CLOSE( g_fMap );

      { Paint map }
      CLRSCR;
      GOTOXY( 1, 1 );
      FOR y := 1 TO MAP_HEIGHT DO
        FOR x := 1 TO MAP_WIDTH DO
          WRITE( MyMap.chMap[ x, y ] );

      { Create snakes }
      iSnakeIndex := 1;
      T1Snake := FirstSnake;
      WHILE ( T1Snake <> NIL ) DO
        BEGIN
          IF ( iRound > 1 ) THEN
            T1Snake^.wSnake.kill
          ELSE
            T1Snake^.wSnake.iPoints := 0;

          IF NOT T1Snake^.wSnake.create( MyMap.Routes[ iSnakeIndex + iRouteOffset ], MyMap.chMap ) THEN
            BEGIN
              { Kill all snakes created before }
              T2Snake := FirstSnake;
              WHILE ( T2Snake <> NIL ) AND ( T2Snake <> T1Snake ) DO
                BEGIN
                  T2Snake^.wSnake.kill;
                  T2Snake := T2Snake^.pNext;

                END;

              game_error( 'Could not create snake #' + chr( 48 + iSnakeIndex ) );

              EXIT;

            END;

          INC( iSnakeIndex );
          T1Snake := T1Snake^.pNext;

        END;

      { Paint snake bodies }
      iSnakeIndex := 1;
      T1Snake := FirstSnake;
      WHILE ( T1Snake <> NIL ) DO
        BEGIN
          TEXTCOLOR( g_iSnakeColor[ iSnakeIndex ] );

          T0Slice := T1Snake^.wSnake.pFoot;
          WHILE ( T0Slice <> T1Snake^.wSnake.pHead ) DO
            BEGIN
              T1Snake^.wSnake.paint_body( T0Slice );
              T0Slice := T0Slice^.pPrev;

            END;
          T1Snake^.wSnake.paint_head;

          INC( iSnakeIndex );
          T1Snake := T1Snake^.pNext;

        END;
      TEXTCOLOR( LIGHTGRAY );

      { Add cookies }
      IF ( g_Settings.iPlayers = 1 ) OR ( g_Settings.iMode <> MODE_BATTLE ) THEN
        BEGIN
          IF ( g_Settings.iPlayers = 1 ) OR ( g_Settings.iMultiCookies = 0 ) THEN
            iCookieCounter := MyMap.iInitialCookies
          ELSE
            iCookieCounter := g_Settings.iMultiCookies;

          WHILE ( iCookieCounter > 0 ) DO
            BEGIN
              DEC( iCookieCounter );
              IF NOT add_cookie( MyMap.chMap ) THEN
                BEGIN
                  game_error( 'Cookie could not be placed' );
                  EXIT;

                END;

            END; { WHILE }

        END; { IF }

      status_bar( FirstSnake );


      iStepCounter := 0;
      REPEAT { ONE ROUND ---------------------------------------------------------------------------------------------- }

        { Step counter / time growth }
        IF ( g_Settings.iPlayers > 1 ) THEN
          BEGIN
            INC( iStepCounter );
            IF ( g_Settings.iMode = MODE_BATTLE ) AND ( iStepCounter = 5 ) THEN
              BEGIN
                iStepCounter := 1;

                T1Snake := FirstSnake;
                WHILE ( T1Snake <> NIL ) DO
                  BEGIN
                    INC( T1Snake^.wSnake.iGrowBody );
                    T1Snake := T1Snake^.pNext;

                  END;

              END; { IF }

          END; { IF }

        { Move feet }
        T1Snake := FirstSnake;
        WHILE ( T1Snake <> NIL ) DO
          BEGIN
            bTemp := ( ( g_Settings.iPlayers = 1 ) OR ( g_Settings.iMode <> MODE_BATTLE ) );
            IF ( T1Snake^.wSnake.move_foot( MyMap.chMap, bTemp ) ) THEN
              T1Snake^.wSnake.paint_foot
            ELSE
              bNewScores := TRUE;

            T1Snake := T1Snake^.pNext;

          END;

        { Move heads }
        T1Snake := FirstSnake;
        WHILE ( T1Snake <> NIL ) DO
          BEGIN
            IF NOT T1Snake^.wSnake.bCrash THEN
              BEGIN
                IF NOT ( T1Snake^.wSnake.move_head( MyMap.chMap ) ) THEN
                  BEGIN
                    { Head of a snake that has already moved? / Multi-crash? }
                    T2Snake := FirstSnake;
                    WHILE ( T2Snake <> NIL ) AND ( T2Snake <> T1Snake ) DO
                      BEGIN
                        IF ( T2Snake^.wSnake.pHead^.x = T1Snake^.wSnake.pHead^.x ) THEN
                          IF ( T2Snake^.wSnake.pHead^.y = T1Snake^.wSnake.pHead^.y ) THEN
                            T2Snake^.wSnake.bCrash := TRUE;

                        T2Snake := T2Snake^.pNext;

                      END; { WHILE }

                    bCrash := TRUE;

                  END { IF }
                ELSE
                  BEGIN
                    { COOKIE or FREE }
                    IF T1Snake^.wSnake.bCookie THEN
                      BEGIN
                        IF ( g_Settings.iMode = MODE_MEAN ) AND ( g_Settings.iPlayers > 1 ) THEN
                          BEGIN
                            { GROW OTHERS }
                            T1Snake^.wSnake.iGrowPoints := T1Snake^.wSnake.iGrowPoints + DEFAULT_GROW;

                            T2Snake := FirstSnake;
                            WHILE ( T2Snake <> NIL ) DO
                              BEGIN
                                IF ( T2Snake <> T1Snake ) THEN
                                  T2Snake^.wSnake.iGrowBody := T2Snake^.wSnake.iGrowBody + DEFAULT_GROW;
                                T2Snake := T2Snake^.pNext;

                              END;

                          END
                        ELSE
                          BEGIN
                            { GROW THIS SNAKE }
                            T1Snake^.wSnake.iGrowPoints := T1Snake^.wSnake.iGrowPoints + DEFAULT_GROW;
                            T1Snake^.wSnake.iGrowBody := T1Snake^.wSnake.iGrowBody + DEFAULT_GROW;

                          END;

                        { ADD NEW COOKIE }
                        IF MyMap.bRefreshCookies THEN
                          add_cookie( MyMap.chMap );
                        T1Snake^.wSnake.bCookie := FALSE;

                      END;

                  END;

              END;

            T1Snake := T1Snake^.pNext;

          END;

        { Paint head and first body }
        iSnakeIndex := 1;
        T1Snake := FirstSnake;
        WHILE ( T1Snake <> NIL ) DO
          BEGIN
            TEXTCOLOR( g_iSnakeColor[ iSnakeIndex ] );
            T1Snake^.wSnake.paint_head;

            { Only paint first body if no other snake crashed into it }
            { Otherwise we'd remove the 'X'... }
            IF NOT bCrash THEN
              T1Snake^.wSnake.paint_body( T1Snake^.wSnake.pHead^.pNext )
            ELSE
              BEGIN
                bAllowPaint := TRUE;

                T2Snake := FirstSnake;
                WHILE ( T2Snake <> NIL ) AND ( T2Snake <> T1Snake ) AND bAllowPaint DO
                  BEGIN
                    IF ( T2Snake^.wSnake.pHead^.x = T1Snake^.wSnake.pHead^.pNext^.x ) THEN
                      IF ( T2Snake^.wSnake.pHead^.y = T1Snake^.wSnake.pHead^.pNext^.y ) THEN
                        bAllowPaint := FALSE;

                    T2Snake := T2Snake^.pNext;

                  END;

                IF bAllowPaint THEN
                  T1Snake^.wSnake.paint_body( T1Snake^.wSnake.pHead^.pNext );

              END;

            INC( iSnakeIndex );
            T1Snake := T1Snake^.pNext;

          END;
        TEXTCOLOR( LIGHTGRAY );

        { Refresh status bar }
        IF bNewScores AND ( ( g_Settings.iMode <> MODE_BATTLE ) OR ( g_Settings.iPlayers = 1 ) ) THEN
          BEGIN
            status_bar( FirstSnake );
            bNewScores := FALSE;

          END;

        { Gamespeed & keystrokes }
        iDelayCounter := iDelayTimes;
        WHILE ( iDelayCounter > 0 ) DO
          BEGIN
            DEC( iDelayCounter );

            { Keystrokes }
            WHILE KEYPRESSED DO
              BEGIN
                chKey := READKEY;

                IF ( chKey = KEY_ESCAPE ) THEN
                  bEscape := TRUE

                ELSE IF ( chKey = KEY_NULL ) THEN
                  { Just skip }

                ELSE
                  BEGIN
                    iSnakeIndex := 1;
                    T1Snake := FirstSnake;
                    bFound := FALSE;
                    WHILE ( T1Snake <> NIL ) AND NOT bFound DO
                      BEGIN
                        iControlIndex := 0;
                        WHILE ( iControlIndex < 4 ) AND NOT bFound DO
                          BEGIN
                            IF ( chKey = g_Settings.Controls[ iControlIndex + ( iSnakeIndex - 1 ) * 4 ] ) THEN
                              BEGIN
                                bFound := TRUE;

                                IF NOT T1Snake^.wSnake.bDirChanged THEN
                                  BEGIN
                                    IF ( ( T1Snake^.wSnake.iDir XOR iControlIndex ) AND 1 = 1 ) THEN
                                      BEGIN
                                        { Change direction }
                                        T1Snake^.wSnake.iDir := iControlIndex;
                                        T1Snake^.wSnake.bDirChanged := TRUE;

                                      END;

                                  END; { IF }

                              END; { IF }

                            INC( iControlIndex );

                          END; { WHILE }

                        INC( iSnakeIndex );
                        T1Snake := T1Snake^.pNext;

                      END; { WHILE }

                  END; { IF }

              END; { WHILE }

            { Gamespeed }
            DELAY( 10 );

          END; { FOR }

        { bDirChanged := FALSE for all }
        T1Snake := FirstSnake;
        WHILE ( T1Snake <> NIL ) DO
          BEGIN
            T1Snake^.wSnake.bDirChanged := FALSE;
            T1Snake := T1Snake^.pNext;

          END;

      UNTIL bEscape OR bCrash;

      IF bCrash THEN
        BEGIN
          IF ( g_Settings.iPlayers = 1 ) THEN
            BEGIN
              { Single player }
              bDone := TRUE;
              game_over_single( FirstSnake^.wSnake.iPoints );

            END
          ELSE
            BEGIN
              { Multiplayer }
              game_over_multi( FirstSnake, iRound, bEscape );

            END;

        END; { IF }

      INC( iRound );

    UNTIL bEscape OR ( g_Settings.iPlayers = 1 ) OR ( iRound > g_iRounds[ g_Settings.iRoundIndex ] );

    { Kill snakes }
    T1Snake := FirstSnake;
    WHILE ( T1Snake <> NIL ) DO
      BEGIN
        T1Snake^.wSnake.kill;
        FirstSnake := T1Snake^.pNext;
        DISPOSE( T1Snake );
        T1Snake := FirstSnake;

      END;

    res_small;

  END;


{ ---------------------------------------------------------------------------------------------------- check_maps ----- }
FUNCTION check_maps : BYTE; { Check map database }
  BEGIN
    {$I-}
    RESET( g_fMap );
    {$I+}

    IF ( IORESULT <> 0 ) THEN
      check_maps := 0

    ELSE
      BEGIN
        { TODO: check map
          gleich lang/
          alles snakefelder frei/
            x prozent frei (+cookies!!!)
          }
          { truncate invalid }
        check_maps := FILESIZE( g_fMap );
        CLOSE( g_fMap );

      END;

  END;


{ ---------------------------------------------------------------------------------------------------------- MAIN ----- }
BEGIN
  { We need random cookies }
  RANDOMIZE;


  { Init colors }
  g_iSnakeColor[ 1 ] := WHITE;
  g_iSnakeColor[ 2 ] := RED;
  g_iSnakeColor[ 3 ] := GREEN;
  g_iSnakeColor[ 4 ] := BLUE;

  g_sSnakeColorName[ 1 ] := 'White';
  g_sSnakeColorName[ 2 ] := 'Red';
  g_sSnakeColorName[ 3 ] := 'Green';
  g_sSnakeColorName[ 4 ] := 'Blue';


  { Check commandline paramameters }
  g_bSkipIntro := FALSE;
  FOR g_iParamIndex := 0 TO PARAMCOUNT DO
    BEGIN
      IF ( PARAMSTR( g_iParamIndex ) = '-NOINTRO' ) OR ( PARAMSTR( g_iParamIndex ) = '-nointro' ) THEN
        g_bSkipIntro := TRUE;

    END;


  { Load settings }
  IF read_settings THEN
    WRITELN( 'Settings loaded' )
  ELSE
    WRITELN( 'Default settings loaded' );


  { Load highscores }
  IF read_scores THEN
    WRITELN( 'Highscores loaded' )
  ELSE
    WRITELN( 'No highscores found' );


  { Check map database }
  ASSIGN( g_fMap, FILE_MAPS );
  g_iMaps := check_maps;
  IF ( g_iMaps > 0 ) THEN
    WRITE( g_iMaps, ' maps found' )
  ELSE
    BEGIN
      WRITE( 'ERROR: No maps found' );
      EXIT;

    END;


  { Give time to read output }
  DELAY( 1200 );


  { Small resolution }
  res_small;


  { Show intro }
  IF NOT g_bSkipIntro THEN
    intro;


  { MAINLOOP }
  g_bQuit := FALSE;
  REPEAT
    IF menu( PAGE_MAIN ) THEN
      game
    ELSE
      g_bQuit := TRUE;

  UNTIL g_bQuit;


  { Save settings }
  save_settings;


  { Save highscores }
  IF g_bNewScores THEN
    save_scores;


  { Restore resolution }
  res_def;

END.