{

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
    *                                                                 *
    *   Copyrightcillas 1.0.3                                         *
    *   Copyright  2004  Florian Oldach <oldach@hartwork.org>        *
    *                     Sebastian Pipping <pipping@hartwork.org>    *
    *                                                                 *
    *   --> http://www.hartwork.org                                   *
    *                                                                 *
    *                                                                 *
    *   This source code is released under LGPL.                      *
    *   See LGPL.txt for details.                        2004-09-01   *
    *                                                                 *
    \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

}

UNIT crc_menu;

{ ----------------------------------------------------------------------------------------------------- INTERFACE ----- }
INTERFACE

  USES
    crt,
    crc_std,
    crc_scor,
    crc_set;

  CONST
    INDEX_MAIN_START             = 0;
    INDEX_MAIN_SETTINGS          = 1;
    INDEX_MAIN_SCORES            = 2;
    INDEX_MAIN_CREDITS           = 3;
    INDEX_MAIN_QUIT              = 4;

    INDEX_SETTINGS_GAME          = 0;
    INDEX_SETTINGS_MULTIPLAYER   = 1;
    INDEX_SETTINGS_CONTROLS      = 2;
    INDEX_SETTINGS_BACK          = 3;

    INDEX_SETTINGS_GAME_PLAYERS  = 0;
    INDEX_SETTINGS_GAME_STYLE    = 1;
    INDEX_SETTINGS_GAME_MAP      = 2;
    INDEX_SETTINGS_GAME_SPEED    = 3;
    INDEX_SETTINGS_GAME_DONE     = 4;

    INDEX_SETTINGS_MULTIPLAYER_MODE     = 0;
    INDEX_SETTINGS_MULTIPLAYER_ROUNDS   = 1;
    INDEX_SETTINGS_MULTIPLAYER_COOKIES  = 2;
    INDEX_SETTINGS_MULTIPLAYER_DONE     = 3;


    PAGE_START                   = INDEX_MAIN_START;
    PAGE_SETTINGS                = INDEX_MAIN_SETTINGS;
    PAGE_SCORES                  = INDEX_MAIN_SCORES;
    PAGE_CREDITS                 = INDEX_MAIN_CREDITS;
    PAGE_QUIT                    = INDEX_MAIN_QUIT;
    PAGE_MAIN                    = 5;
    PAGE_SETTINGS_GAME           = 6;
    PAGE_SETTINGS_MULTIPLAYER    = 7;
    PAGE_SETTINGS_CONTROLS       = 8;
    PAGE_SETTINGS_BACK           = 9;

    WORK_TOP                     = 12;

  FUNCTION  menu( iPage : BYTE ) : BOOLEAN;


{ ------------------------------------------------------------------------------------------------ IMPLEMENTATION ----- }
IMPLEMENTATION

  TYPE
    pMapList = ^tMapList;
    tMapList = RECORD
      iFilePos : BYTE;
      sTitle   : STRING[ MAX_NAME_LENGTH ];
      pPrev    : pMapList;
      pNext    : pMapList;

    END;


{ --------------------------------------------------------------------------------------------------- window_work ----- }
  PROCEDURE window_work;
    BEGIN
      WINDOW( 1, WORK_TOP, 80, WORK_TOP + 10 ); { 11 lines }

    END;


{ ---------------------------------------------------------------------------------------------------- arrow_move ----- }
  PROCEDURE arrow_move( iOldY, iNewY : BYTE );
    BEGIN
      GOTOXY( 1, iOldY );
      WRITE( ASCII_SPACE );

      GOTOXY( 1, iNewY );
      WRITE( ASCII_ARROW_RIGHT );

    END;


{ ----------------------------------------------------------------------------------------------------- main_info ----- }
  PROCEDURE main_info;
    CONST
      MAIN_INFO_TOP = 24;
      MAIN_INFO_TEXT = 'Copyright  2004 Florian Oldach, Sebastian Pipping';

    BEGIN
      GOTOXY( ( SCREEN_WIDTH - LENGTH( MAIN_INFO_TEXT ) ) DIV 2 + 1, MAIN_INFO_TOP );
      WRITE( MAIN_INFO_TEXT );

    END;


{ ----------------------------------------------------------------------------------------------------- main_logo ----- }
  PROCEDURE main_logo;
    BEGIN
      GOTOXY( 1, 3 );
      WRITELN( '                                                      ' );
      WRITELN( '                                                        ' );
      WRITELN( '                            ' );
      WRITELN( '                                              ' );
      WRITELN( '                          ' );
      WRITE(   '' );
      WRITELN( '                                ' );

    END;


{ ----------------------------------------------------------------------------------------------------- menu_main ----- }
  FUNCTION menu_main( iIndex : BYTE ) : BYTE;
    VAR
      iCur, iPrev : BYTE;
      bQuit : BOOLEAN;
      chKey : CHAR;
      iFrameLeft : BYTE;
      iY : ARRAY[ INDEX_MAIN_START..INDEX_MAIN_QUIT ] OF BYTE;

    BEGIN
      iY[ INDEX_MAIN_START ]     := 1;
      iY[ INDEX_MAIN_SETTINGS ]  := 3;
      iY[ INDEX_MAIN_SCORES ]    := 4;
      iY[ INDEX_MAIN_CREDITS ]   := 5;
      iY[ INDEX_MAIN_QUIT ]      := 7;

      iCur := iIndex;
      iPrev := iCur;

      CLRSCR;
      iFrameLeft := frame_center( 18, 9, TRUE );
      WINDOW( iFrameLeft + 4, WORK_TOP + 1, iFrameLeft + 14, WORK_TOP + 7 );
      WRITELN( 'Start' );
      WRITELN;
      WRITELN( 'Settings' );
      WRITELN( 'Highscores' );
      WRITELN( 'Credits' );
      WRITELN;
      WRITE( 'Quit' );

      WINDOW( iFrameLeft + 1, WORK_TOP + 1, iFrameLeft + 2, WORK_TOP + 7 );
      arrow_move( iY[ iPrev ], iY[ iCur ] );

      bQuit := FALSE;
      REPEAT
        chKey := READKEY;
        CASE chKey OF

          KEY_UP:
            BEGIN
              iPrev := iCur;
              iCur := ( iCur + 4 ) MOD 5;
              arrow_move( iY[ iPrev ], iY[ iCur ] );

            END;

          KEY_DOWN:
            BEGIN
              iPrev := iCur;
              iCur := ( iCur + 1 ) MOD 5;
              arrow_move( iY[ iPrev ], iY[ iCur ] );

            END;

          KEY_RETURN, KEY_SPACE:
            bQuit := TRUE;

          KEY_ESCAPE:
            BEGIN
              iCur := INDEX_MAIN_QUIT;
              bQuit := TRUE;

            END;

        END; { CASE }

      UNTIL bQuit;

      window_work;
      menu_main := iCur;

    END;


{ ------------------------------------------------------------------------------------------------- menu_settings ----- }
  FUNCTION menu_settings( iIndex : BYTE ) : BYTE;
    VAR
      iCur, iPrev : BYTE;
      bQuit : BOOLEAN;
      chKey : CHAR;
      iFrameLeft : BYTE;
      iY : ARRAY[ INDEX_SETTINGS_GAME..INDEX_SETTINGS_BACK ] OF BYTE;

    BEGIN
      iY[ INDEX_SETTINGS_GAME ]         := 1;
      iY[ INDEX_SETTINGS_MULTIPLAYER ]  := 2;
      iY[ INDEX_SETTINGS_CONTROLS ]     := 3;
      iY[ INDEX_SETTINGS_BACK ]         := 5;

      iCur := iIndex;
      iPrev := iCur;

      CLRSCR;
      iFrameLeft := frame_center( 19, 7, TRUE );
      WINDOW( iFrameLeft + 4, WORK_TOP + 1, iFrameLeft + 15, WORK_TOP + 5 );
      WRITELN( 'Game' );
      WRITELN( 'Multiplayer' );
      WRITELN( 'Controls' );
      WRITELN;
      WRITE( 'Back' );

      WINDOW( iFrameLeft + 1, WORK_TOP + 1, iFrameLeft + 2, WORK_TOP + 5 );
      arrow_move( iY[ iPrev ], iY[ iCur ] );

      bQuit := FALSE;
      REPEAT
        chKey := READKEY;
        CASE chKey OF

          KEY_UP:
            BEGIN
              iPrev := iCur;
              iCur := ( iCur + 3 ) MOD 4;
              arrow_move( iY[ iPrev ], iY[ iCur ] );

            END;

          KEY_DOWN:
            BEGIN
              iPrev := iCur;
              iCur := ( iCur + 1 ) MOD 4;
              arrow_move( iY[ iPrev ], iY[ iCur ] );

            END;

          KEY_RETURN, KEY_SPACE:
            bQuit := TRUE;

          KEY_ESCAPE:
            BEGIN
              iCur := INDEX_SETTINGS_BACK;
              bQuit := TRUE;

            END;

        END; { CASE }

      UNTIL bQuit;

      window_work;
      menu_settings := iCur;

    END;


{ ------------------------------------------------------------------------------------ item_settings_game_players ----- }
  PROCEDURE item_settings_game_players;
    VAR
      i : BYTE;

    BEGIN
      GOTOXY( 14, 1 );
      FOR i := 1 TO MAX_PLAYERS DO
        BEGIN
          IF ( i = g_Settings.iPlayers ) THEN
            TEXTCOLOR( LIGHTGRAY )
          ELSE
            TEXTCOLOR( DARKGRAY );

          WRITE( i, ' ' );

        END;
      TEXTCOLOR( LIGHTGRAY );

    END;


{ -------------------------------------------------------------------------------------- item_settings_game_style ----- }
  PROCEDURE item_settings_game_style;
    BEGIN
      GOTOXY( 14, 2 );
      IF ( g_Settings.bSolidSnakes ) THEN
        BEGIN
          WRITE( 'Solid' );
          TEXTCOLOR( DARKGRAY );
          WRITE( ' / Bones' );
          TEXTCOLOR( LIGHTGRAY );

        END
      ELSE
        BEGIN
          TEXTCOLOR( DARKGRAY );
          WRITE( 'Solid / ' );
          TEXTCOLOR( LIGHTGRAY );
          WRITE( 'Bones' );

        END;

    END;


{ ---------------------------------------------------------------------------------------- item_settings_game_map ----- }
  PROCEDURE item_settings_game_map( MyList : pMapList );
    VAR
      i : BYTE;

    BEGIN
      GOTOXY( 14, 3 );
      WRITE( MyList^.sTitle );
      FOR i := 1 TO 20 - LENGTH( MyList^.sTitle ) DO
        WRITE( ASCII_SPACE );

      GOTOXY( 36, 3 );
      IF ( g_Settings.iMap < 9 ) THEN
        WRITE( '0' );
      WRITE( g_Settings.iMap + 1 );

    END;


{ -------------------------------------------------------------------------------------- item_settings_game_speed ----- }
  PROCEDURE item_settings_game_speed;
    VAR
      i : BYTE;

    BEGIN
      GOTOXY( 15, 4 );
      FOR i := 1 TO g_Settings.iSpeed DO
        WRITE( ASCII_BLOCK_MID );
      TEXTCOLOR( DARKGRAY );
      FOR i := g_Settings.iSpeed + 1 TO 10 DO
        WRITE( ASCII_BLOCK_MID );
      TEXTCOLOR( LIGHTGRAY );

      GOTOXY( 32, 4 );
      CASE g_Settings.iSpeed OF
        1..4:
          WRITE( '  Slow' );

        5..7:
          WRITE( 'Medium' );

        8..10:
          WRITE( '  Fast' );

      END;

    END;


{ -------------------------------------------------------------------------------------------- menu_settings_game ----- }
  PROCEDURE menu_settings_game;
    VAR
      iFrameLeft   : BYTE;
      iY           : ARRAY[ INDEX_SETTINGS_GAME_PLAYERS..INDEX_SETTINGS_GAME_DONE ] OF BYTE;

      iCur, iPrev  : BYTE;
      bQuit        : BOOLEAN;
      chKey        : CHAR;

      MyFile       : tFileMap;
      MyMap        : tMapEx;

      MyList       : pMapList;
      TemList      : pMapList;
      pFirst, pLast, pSel : pMapList;

      i            : BYTE;
      iMapMax      : BYTE;

      iOldMap      : BYTE;
      iMapCounter  : BYTE;

    BEGIN
      ASSIGN( MyFile, FILE_MAPS );

      {$I-}
      RESET( MyFile );
      {$I+}

      pFirst := NIL;
      pSel := NIL;
      MyList := NIL;
      TemList := NIL;

      IF ( IORESULT = 0 ) THEN
        BEGIN
          iMapMax := FILESIZE( MyFile ) - 1;
          IF ( iMapMax >= 0 ) THEN
            BEGIN
              IF ( g_Settings.iMap > iMapMax ) THEN
                g_Settings.iMap := 0;

              { First }
              READ( MyFile, MyMap );
              NEW( MyList );
              MyList^.sTitle := MyMap.sTitle;
              MyList^.pPrev := NIL;
              pFirst := MyList;
              IF ( g_Settings.iMap = 0 ) THEN
                pSel := MyList;

              { Middle }
              FOR i := 1 TO ( iMapMax - 1 ) DO
                BEGIN
                  READ( MyFile, MyMap );
                  NEW( TemList );
                  TemList^.sTitle := MyMap.sTitle;
                  TemList^.pPrev := MyList;
                  MyList^.pNext := TemList;
                  MyList := TemList;
                  IF ( g_Settings.iMap = i ) THEN
                    pSel := MyList;

                END;

              { Last }
              IF ( iMapMax > 1 ) THEN
                BEGIN
                  READ( MyFile, MyMap );
                  NEW( TemList );
                  TemList^.sTitle := MyMap.sTitle;
                  TemList^.pPrev := MyList;
                  TemList^.pNext := NIL;
                  MyList^.pNext := TemList;
                  pLast := TemList;
                  IF ( g_Settings.iMap = iMapMax ) THEN
                    pSel := TemList;

                END
              ELSE
                BEGIN
                  MyList^.pNext := NIL;
                  pLast := MyList;
                  pSel := MyList;

                END;

              CLOSE( MyFile );

          END;

        END;

      iY[ INDEX_SETTINGS_GAME_PLAYERS ]  := 1;
      iY[ INDEX_SETTINGS_GAME_STYLE ]    := 2;
      iY[ INDEX_SETTINGS_GAME_MAP ]      := 3;
      iY[ INDEX_SETTINGS_GAME_SPEED ]    := 4;
      iY[ INDEX_SETTINGS_GAME_DONE ]     := 6;

      iCur := INDEX_SETTINGS_GAME_PLAYERS;
      iPrev := iCur;

      CLRSCR;
      iFrameLeft := frame_center( 40, 8, TRUE );
      WINDOW( iFrameLeft + 4, WORK_TOP + 1, iFrameLeft + 38, WORK_TOP + 6 );
      WRITELN( 'Players ' );
      WRITELN( 'Style   ' );
      WRITELN( 'Map      xxxxxXXXXXxxxxxXXXXX #xx' );
      WRITELN( 'Speed    [          ]' );
      WRITELN( '        ' );
      WRITE(   'Done    ' );

      WINDOW( iFrameLeft + 1, WORK_TOP + 1, iFrameLeft + 38, WORK_TOP + 6 );
      arrow_move( iY[ iPrev ], iY[ iCur ] );

      item_settings_game_players;
      item_settings_game_style;
      item_settings_game_map( pSel );
      item_settings_game_speed;

      bQuit := FALSE;
      REPEAT
        chKey := READKEY;
        CASE chKey OF

          KEY_UP:
            BEGIN
              iPrev := iCur;
              iCur := ( iCur + 4 ) MOD 5;
              arrow_move( iY[ iPrev ], iY[ iCur ] );

            END;

          KEY_DOWN:
            BEGIN
              iPrev := iCur;
              iCur := ( iCur + 1 ) MOD 5;
              arrow_move( iY[ iPrev ], iY[ iCur ] );

            END;

          KEY_LEFT:
            BEGIN
              CASE iCur OF
                INDEX_SETTINGS_GAME_PLAYERS:
                  BEGIN
                    IF ( g_Settings.iPlayers > 1 ) THEN
                      BEGIN
                        DEC( g_Settings.iPlayers );
                        item_settings_game_players;

                      END;

                  END;

                INDEX_SETTINGS_GAME_STYLE:
                  BEGIN
                    IF NOT g_Settings.bSolidSnakes THEN
                      BEGIN
                        g_Settings.bSolidSnakes := TRUE;
                        item_settings_game_style;

                      END;

                  END;

                INDEX_SETTINGS_GAME_MAP:
                  BEGIN
                    IF ( g_Settings.iMap > 0 ) THEN
                      BEGIN
                        DEC( g_Settings.iMap );
                        pSel := pSel^.pPrev;
                        item_settings_game_map( pSel );

                      END;

                  END;

                INDEX_SETTINGS_GAME_SPEED:
                  BEGIN
                    IF ( g_Settings.iSpeed > 1 ) THEN
                      BEGIN
                        DEC( g_Settings.iSpeed );
                        item_settings_game_speed;

                      END;

                  END;

              END; { CASE }

            END;

          KEY_RIGHT:
            BEGIN
              CASE iCur OF
                INDEX_SETTINGS_GAME_PLAYERS:
                  BEGIN
                    IF ( g_Settings.iPlayers < MAX_PLAYERS ) THEN
                      BEGIN
                        INC( g_Settings.iPlayers );
                        item_settings_game_players;

                      END;

                  END;

                INDEX_SETTINGS_GAME_STYLE:
                  BEGIN
                    IF g_Settings.bSolidSnakes THEN
                      BEGIN
                        g_Settings.bSolidSnakes := FALSE;
                        item_settings_game_style;

                      END;

                  END;

                INDEX_SETTINGS_GAME_MAP:
                  BEGIN
                    IF ( g_Settings.iMap < iMapMax ) THEN
                      BEGIN
                        INC( g_Settings.iMap );
                        pSel := pSel^.pNext;
                        item_settings_game_map( pSel );

                      END;

                  END;

                INDEX_SETTINGS_GAME_SPEED:
                  BEGIN
                    IF ( g_Settings.iSpeed < 10 ) THEN
                      BEGIN
                        INC( g_Settings.iSpeed );
                        item_settings_game_speed;

                      END;

                  END;

              END; { CASE }

            END;

          KEY_HOME:
            BEGIN
              CASE iCur OF
                INDEX_SETTINGS_GAME_PLAYERS:
                  BEGIN
                    IF ( g_Settings.iPlayers > 1 ) THEN
                      BEGIN
                        g_Settings.iPlayers := 1;
                        item_settings_game_players;

                      END;

                    END;

                INDEX_SETTINGS_GAME_STYLE:
                  BEGIN
                    IF NOT g_Settings.bSolidSnakes THEN
                      BEGIN
                        g_Settings.bSolidSnakes := TRUE;
                        item_settings_game_style;

                      END;

                    END;

                INDEX_SETTINGS_GAME_MAP:
                  BEGIN
                    IF ( g_Settings.iMap > 0 ) THEN
                      BEGIN
                        g_Settings.iMap := 0;
                        pSel := pFirst;
                        item_settings_game_map( pSel );

                      END;

                  END;

                INDEX_SETTINGS_GAME_SPEED:
                  BEGIN
                    IF ( g_Settings.iSpeed > 1 ) THEN
                      BEGIN
                        g_Settings.iSpeed := 1;
                        item_settings_game_speed;

                      END;

                  END;

              END; { CASE }

            END;

          KEY_END:
            BEGIN
              CASE iCur OF
                INDEX_SETTINGS_GAME_PLAYERS:
                  BEGIN
                    IF ( g_Settings.iPlayers < MAX_PLAYERS ) THEN
                      BEGIN
                        g_Settings.iPlayers := MAX_PLAYERS;
                        item_settings_game_players;

                      END;

                  END;

                INDEX_SETTINGS_GAME_STYLE:
                  BEGIN
                    IF g_Settings.bSolidSnakes THEN
                      BEGIN
                        g_Settings.bSolidSnakes := FALSE;
                        item_settings_game_style;

                      END;

                  END;

                INDEX_SETTINGS_GAME_MAP:
                  BEGIN
                    IF ( g_Settings.iMap < iMapMax ) THEN
                      BEGIN
                        g_Settings.iMap := iMapMax;
                        pSel := pLast;
                        item_settings_game_map( pSel );

                      END;

                    END;

                INDEX_SETTINGS_GAME_SPEED:
                  BEGIN
                    IF ( g_Settings.iSpeed < 10 ) THEN
                      BEGIN
                        g_Settings.iSpeed := 10;
                        item_settings_game_speed;

                      END;

                  END;

              END; { CASE }

            END;

          KEY_PAGEUP:
            BEGIN
              CASE iCur OF
                INDEX_SETTINGS_GAME_PLAYERS:
                  BEGIN
                    IF ( g_Settings.iPlayers > 1 ) THEN
                      BEGIN
                        g_Settings.iPlayers := 1;
                        item_settings_game_players;

                      END;

                    END;

                INDEX_SETTINGS_GAME_STYLE:
                  BEGIN
                    IF NOT g_Settings.bSolidSnakes THEN
                      BEGIN
                        g_Settings.bSolidSnakes := TRUE;
                        item_settings_game_style;

                      END;

                    END;

                INDEX_SETTINGS_GAME_MAP:
                  BEGIN
                    IF ( g_Settings.iMap > 0 ) THEN
                      BEGIN
                        iOldMap := g_Settings.iMap;
                        IF ( g_Settings.iMap > 4 ) THEN
                          g_Settings.iMap := g_Settings.iMap - ( g_Settings.iMap MOD 5 ) - 1
                        ELSE
                          g_Settings.iMap := 0;
                        
                        iMapCounter := iOldMap - g_Settings.iMap;
                        WHILE ( pSel^.pPrev <> NIL ) AND ( iMapCounter > 0 ) DO
                          BEGIN
                            pSel := pSel^.pPrev;
                            DEC( iMapCounter );

                          END;
                        item_settings_game_map( pSel );

                      END;

                  END;

                INDEX_SETTINGS_GAME_SPEED:
                  BEGIN
                    IF ( g_Settings.iSpeed > 1 ) THEN
                      BEGIN
                        IF ( g_Settings.iSpeed > 6 ) THEN
                          g_Settings.iSpeed := g_Settings.iSpeed - ( ( g_Settings.iSpeed - 1 ) MOD 5 ) - 1
                        ELSE
                          g_Settings.iSpeed := 1;
                        item_settings_game_speed;

                      END;

                  END;

              END; { CASE }

            END;

          KEY_PAGEDOWN:
            BEGIN
              CASE iCur OF
                INDEX_SETTINGS_GAME_PLAYERS:
                  BEGIN
                    IF ( g_Settings.iPlayers < MAX_PLAYERS ) THEN
                      BEGIN
                        g_Settings.iPlayers := MAX_PLAYERS;
                        item_settings_game_players;

                      END;

                    END;

                INDEX_SETTINGS_GAME_STYLE:
                  BEGIN
                    IF g_Settings.bSolidSnakes THEN
                      BEGIN
                        g_Settings.bSolidSnakes := FALSE;
                        item_settings_game_style;

                      END;

                    END;

                INDEX_SETTINGS_GAME_MAP:
                  BEGIN
                    IF ( g_Settings.iMap < iMapMax ) THEN
                      BEGIN
                        iOldMap := g_Settings.iMap;
                        g_Settings.iMap := g_Settings.iMap + 5 - ( ( g_Settings.iMap + 1 ) MOD 5 );
                        IF ( g_Settings.iMap > iMapMax ) THEN
                          g_Settings.iMap := iMapMax;
                        
                        iMapCounter := g_Settings.iMap - iOldMap;
                        WHILE ( pSel^.pNext <> NIL ) AND ( iMapCounter > 0 ) DO
                          BEGIN
                            pSel := pSel^.pNext;
                            DEC( iMapCounter );

                          END;
                        item_settings_game_map( pSel );

                      END;

                  END;

                INDEX_SETTINGS_GAME_SPEED:
                  BEGIN
                    IF ( g_Settings.iSpeed < 10 ) THEN
                      BEGIN
                        g_Settings.iSpeed := g_Settings.iSpeed + 5 - ( g_Settings.iSpeed MOD 5 );
                        item_settings_game_speed;

                      END;

                  END;

              END; { CASE }

            END;

          KEY_RETURN, KEY_SPACE:
            BEGIN
              IF ( iCur = INDEX_SETTINGS_GAME_DONE ) THEN
                bQuit := TRUE;

            END;

          KEY_ESCAPE:
            BEGIN
              iCur := INDEX_SETTINGS_BACK;
              bQuit := TRUE;

            END;

          'r':
            BEGIN
              IF ( iCur = INDEX_SETTINGS_GAME_MAP ) AND ( iMapMax > 1 ) THEN
                BEGIN
                  { Pick random map }
                  iMapCounter := g_Settings.iMap;
                  REPEAT
                    g_Settings.iMap := RANDOM( iMapMax + 1 );

                  UNTIL ( g_Settings.iMap <> iMapCounter );

                  { Search map name }
                  iMapCounter := g_Settings.iMap;
                  pSel := pFirst;
                  WHILE ( pSel <> NIL ) AND ( iMapCounter > 0 ) DO
                    BEGIN
                      DEC( iMapCounter );
                      pSel := pSel^.pNext;

                    END;

                  item_settings_game_map( pSel );

                END; { IF }

            END;

        END; { CASE }

      UNTIL bQuit;

      { Delete maplist }
      WHILE ( pFirst <> NIL ) DO
        BEGIN
          TemList := pFirst^.pNext;
          DISPOSE( pFirst );
          pFirst := TemList;

        END;

      window_work;

    END;


{ -------------------------------------------------------------------------------- item_settings_multiplayer_mode ----- }
  PROCEDURE item_settings_multiplayer_mode;
    BEGIN
      GOTOXY( 14, 1 );
      CASE g_Settings.iMode OF

        MODE_NORMAL:
          BEGIN
            WRITE( 'Normal' );
            TEXTCOLOR( DARKGRAY );
            WRITE( ' / Battle / Mean' );
            TEXTCOLOR( LIGHTGRAY );

          END;

        MODE_BATTLE:
          BEGIN
            TEXTCOLOR( DARKGRAY );
            WRITE( 'Normal / ' );
            TEXTCOLOR( LIGHTGRAY );
            WRITE( 'Battle' );
            TEXTCOLOR( DARKGRAY );
            WRITE( ' / Mean' );
            TEXTCOLOR( LIGHTGRAY );


          END;

        MODE_MEAN:
          BEGIN
            TEXTCOLOR( DARKGRAY );
            WRITE( 'Normal / Battle / ' );
            TEXTCOLOR( LIGHTGRAY );
            WRITE( 'Mean' );

          END;

      END; { CASE }

    END;


{ ------------------------------------------------------------------------------ item_settings_multiplayer_rounds ----- }
  PROCEDURE item_settings_multiplayer_rounds;
    VAR
      i : BYTE;

    BEGIN
      GOTOXY( 14, 2 );
      FOR i := 1 TO 4 DO
        BEGIN
          IF ( i = g_Settings.iRoundIndex ) THEN
            TEXTCOLOR( LIGHTGRAY )
          ELSE
            TEXTCOLOR( DARKGRAY );

          WRITE( g_iRounds[ i ], ' ' );

        END;
      TEXTCOLOR( LIGHTGRAY );

    END;


{ ----------------------------------------------------------------------------- item_settings_multiplayer_cookies ----- }
  PROCEDURE item_settings_multiplayer_cookies;
    VAR
      i : BYTE;

    BEGIN
      GOTOXY( 14, 3 );
      IF ( g_Settings.iMultiCookies <> 0 ) THEN
        BEGIN
          WRITE( g_Settings.iMultiCookies );
          IF ( g_Settings.iMultiCookies < 10 ) THEN
             WRITE( '  ' )
          ELSE IF ( g_Settings.iMultiCookies < 100 ) THEN
             WRITE( ' ' );
          WRITE( '        ' );

        END
      ELSE
        WRITE( 'Map defined' );

    END;


{ ------------------------------------------------------------------------------------- menu_settings_multiplayer ----- }
  PROCEDURE menu_settings_multiplayer;
    VAR
      iCur, iPrev  : BYTE;
      bQuit        : BOOLEAN;
      chKey        : CHAR;
      iFrameLeft   : BYTE;
      iY           : ARRAY[ INDEX_SETTINGS_MULTIPLAYER_MODE..INDEX_SETTINGS_MULTIPLAYER_DONE ] OF BYTE;

    BEGIN
      iY[ INDEX_SETTINGS_MULTIPLAYER_MODE ]    := 1;
      iY[ INDEX_SETTINGS_MULTIPLAYER_ROUNDS ]  := 2;
      iY[ INDEX_SETTINGS_MULTIPLAYER_COOKIES ] := 3;
      iY[ INDEX_SETTINGS_MULTIPLAYER_DONE ]    := 5;

      iCur := INDEX_SETTINGS_MULTIPLAYER_MODE;
      iPrev := iCur;

      CLRSCR;
      iFrameLeft := frame_center( 38, 7, TRUE );
      WINDOW( iFrameLeft + 4, WORK_TOP + 1, iFrameLeft + 36, WORK_TOP + 5 );
      WRITELN( 'Mode     Normal / Battle / Mean' );
      WRITELN( 'Rounds   1 3 5 9' );
      WRITELN( 'Cookies  4' );
      WRITELN( '        ' );
      WRITE(   'Done    ' );
      WINDOW( iFrameLeft + 1, WORK_TOP + 1, iFrameLeft + 36, WORK_TOP + 5 );
      arrow_move( iY[ iPrev ], iY[ iCur ] );

      item_settings_multiplayer_mode;
      item_settings_multiplayer_rounds;
      item_settings_multiplayer_cookies;

      bQuit := FALSE;
      REPEAT
        chKey := READKEY;
        CASE chKey OF

          KEY_UP:
            BEGIN
              iPrev := iCur;
              iCur := ( iCur + 3 ) MOD 4;
              arrow_move( iY[ iPrev ], iY[ iCur ] );

            END;

          KEY_DOWN:
            BEGIN
              iPrev := iCur;
              iCur := ( iCur + 1 ) MOD 4;
              arrow_move( iY[ iPrev ], iY[ iCur ] );

            END;

          KEY_LEFT:
            BEGIN
              CASE iCur OF

                INDEX_SETTINGS_MULTIPLAYER_MODE:
                  BEGIN
                    IF ( g_Settings.iMode > MODE_NORMAL ) THEN
                      BEGIN
                        DEC( g_Settings.iMode );
                        item_settings_multiplayer_mode;

                      END;

                  END;

                INDEX_SETTINGS_MULTIPLAYER_ROUNDS:
                  BEGIN
                    IF ( g_Settings.iRoundIndex > 1 ) THEN
                      BEGIN
                        DEC( g_Settings.iRoundIndex );
                        item_settings_multiplayer_rounds;

                      END;

                  END;

                INDEX_SETTINGS_MULTIPLAYER_COOKIES:
                  BEGIN
                    IF ( g_Settings.iMultiCookies > 0 ) THEN
                      BEGIN
                        DEC( g_Settings.iMultiCookies );
                        item_settings_multiplayer_cookies;

                      END;

                  END;

              END; { CASE }

            END;

          KEY_RIGHT:
            BEGIN
              CASE iCur OF

                INDEX_SETTINGS_MULTIPLAYER_MODE:
                  BEGIN
                    IF ( g_Settings.iMode < MODE_MEAN ) THEN
                      BEGIN
                        INC( g_Settings.iMode );
                        item_settings_multiplayer_mode;

                      END;

                  END;

                INDEX_SETTINGS_MULTIPLAYER_ROUNDS:
                  BEGIN
                    IF ( g_Settings.iRoundIndex < 4 ) THEN
                      BEGIN
                        INC( g_Settings.iRoundIndex );
                        item_settings_multiplayer_rounds;

                      END;

                  END;

                INDEX_SETTINGS_MULTIPLAYER_COOKIES:
                  BEGIN
                    IF ( g_Settings.iMultiCookies < MAX_COOKIES ) THEN
                      BEGIN
                        INC( g_Settings.iMultiCookies );
                        item_settings_multiplayer_cookies;

                      END;

                  END;

              END; { CASE }

            END;

          KEY_HOME:
            BEGIN
              CASE iCur OF

                INDEX_SETTINGS_MULTIPLAYER_MODE:
                  BEGIN
                    IF ( g_Settings.iMode > MODE_NORMAL ) THEN
                      BEGIN
                        g_Settings.iMode := MODE_NORMAL;
                        item_settings_multiplayer_mode;

                      END;

                  END;

                INDEX_SETTINGS_MULTIPLAYER_ROUNDS:
                  BEGIN
                    IF ( g_Settings.iRoundIndex > 1 ) THEN
                      BEGIN
                        g_Settings.iRoundIndex := 1;
                        item_settings_multiplayer_rounds;

                      END;

                  END;

                INDEX_SETTINGS_MULTIPLAYER_COOKIES:
                  BEGIN
                    IF ( g_Settings.iMultiCookies > 0 ) THEN
                      BEGIN
                        g_Settings.iMultiCookies := 0;
                        item_settings_multiplayer_cookies;

                      END;

                  END;

              END; { CASE }

            END;

          KEY_END:
            BEGIN
              CASE iCur OF

                INDEX_SETTINGS_MULTIPLAYER_MODE:
                  BEGIN
                    IF ( g_Settings.iMode < MODE_MEAN ) THEN
                      BEGIN
                        g_Settings.iMode := MODE_MEAN;
                        item_settings_multiplayer_mode;

                      END;

                  END;

                INDEX_SETTINGS_MULTIPLAYER_ROUNDS:
                  BEGIN
                    IF ( g_Settings.iRoundIndex < 4 ) THEN
                      BEGIN
                        g_Settings.iRoundIndex := 4;
                        item_settings_multiplayer_rounds;

                      END;

                  END;

                INDEX_SETTINGS_MULTIPLAYER_COOKIES:
                  BEGIN
                    IF ( g_Settings.iMultiCookies < MAX_COOKIES ) THEN
                      BEGIN
                        g_Settings.iMultiCookies := MAX_COOKIES;
                        item_settings_multiplayer_cookies;

                      END;

                  END;

              END; { CASE }

            END;

          KEY_PAGEUP:
            BEGIN
              CASE iCur OF

                INDEX_SETTINGS_MULTIPLAYER_MODE:
                  BEGIN
                    IF ( g_Settings.iMode > MODE_NORMAL ) THEN
                      BEGIN
                        g_Settings.iMode := MODE_NORMAL;
                        item_settings_multiplayer_mode;

                      END;

                  END;

                INDEX_SETTINGS_MULTIPLAYER_ROUNDS:
                  BEGIN
                    IF ( g_Settings.iRoundIndex > 1 ) THEN
                      BEGIN
                        g_Settings.iRoundIndex := 1;
                        item_settings_multiplayer_rounds;

                      END;

                  END;

                INDEX_SETTINGS_MULTIPLAYER_COOKIES:
                  BEGIN
                    IF ( g_Settings.iMultiCookies > 0 ) THEN
                      BEGIN
                        IF ( g_Settings.iMultiCookies > 5 ) THEN
                          g_Settings.iMultiCookies := g_Settings.iMultiCookies - ( ( g_Settings.iMultiCookies - 1 ) MOD 5 ) - 1
                        ELSE
                          g_Settings.iMultiCookies := 0;
                        item_settings_multiplayer_cookies;

                      END;

                  END;

              END; { CASE }

            END;

          KEY_PAGEDOWN:
            BEGIN
              CASE iCur OF

                INDEX_SETTINGS_MULTIPLAYER_MODE:
                  BEGIN
                    IF ( g_Settings.iMode < MODE_MEAN ) THEN
                      BEGIN
                        g_Settings.iMode := MODE_MEAN;
                        item_settings_multiplayer_mode;

                      END;

                  END;

                INDEX_SETTINGS_MULTIPLAYER_ROUNDS:
                  BEGIN
                    IF ( g_Settings.iRoundIndex < 4 ) THEN
                      BEGIN
                        g_Settings.iRoundIndex := 4;
                        item_settings_multiplayer_rounds;

                      END;

                  END;

                INDEX_SETTINGS_MULTIPLAYER_COOKIES:
                  BEGIN
                    IF ( g_Settings.iMultiCookies < MAX_COOKIES ) THEN
                      BEGIN
                        IF ( g_Settings.iMultiCookies < ( MAX_COOKIES - 6 ) ) THEN
                          g_Settings.iMultiCookies := g_Settings.iMultiCookies + 5 - ( g_Settings.iMultiCookies MOD 5 )
                        ELSE
                          g_Settings.iMultiCookies := MAX_COOKIES;
                        item_settings_multiplayer_cookies;

                      END;

                  END;

              END; { CASE }

            END;

          KEY_RETURN, KEY_SPACE:
            BEGIN
              IF ( iCur = INDEX_SETTINGS_MULTIPLAYER_DONE ) THEN
                bQuit := TRUE;

            END;

          KEY_ESCAPE:
            BEGIN
              iCur := INDEX_SETTINGS_BACK;
              bQuit := TRUE;

            END;

        END; { CASE }

      UNTIL bQuit;

      window_work;

    END;


{ --------------------------------------------------------------------------------------------------------- ex2in ----- }
  FUNCTION ex2in( iIndex : BYTE; iPlayer : BYTE ) : BYTE;
    BEGIN
      CASE iIndex OF
        0: iIndex := DIR_LEFT;
        1: iIndex := DIR_RIGHT;
        2: iIndex := DIR_UP;
        3: iIndex := DIR_DOWN;

      END;

      iIndex := iIndex + ( iPlayer - 1 ) * 4;
      ex2in := iIndex;

    END;


{ ----------------------------------------------------------------------------------------------------------in2ex ----- }
  FUNCTION in2ex( iIndex : BYTE ) : BYTE;
    BEGIN
      iIndex := iIndex MOD 4;
      CASE iIndex OF
        DIR_LEFT:  iIndex := 0;
        DIR_RIGHT: iIndex := 1;
        DIR_UP:    iIndex := 2;
        DIR_DOWN:  iIndex := 3;

      END;

      in2ex := iIndex;

    END;


{ ---------------------------------------------------------------------------------------------------- key_string ----- }
  FUNCTION key_string( chKey : CHAR ) : STRING;
    BEGIN
      CASE chKey OF
        '0'..'9':		key_string := '    ' + chKey + '    ';
        'a'..'z':		key_string := '    ' + CHR( ORD( chKey ) - 32 ) + '   ';
        KEY_INSERT:		key_string := ' INSERT ';
        KEY_DELETE:		key_string := ' DELETE ';
        KEY_HOME:		key_string := '  HOME  ';
        KEY_END:		key_string := '   END  ';
        KEY_PAGEUP:		key_string := ' PAGEUP ';
        KEY_PAGEDOWN:	key_string := 'PAGEDOWN';
        KEY_LEFT:		key_string := '  LEFT  ';
        KEY_RIGHT:		key_string := '  RIGHT ';
        KEY_UP:			key_string := '   UP   ';
        KEY_DOWN:		key_string := '  DOWN  ';

      ELSE
        				key_string := '  ERROR ';

      END;

    END;


{ ---------------------------------------------------------------------------------------------------- select_key ----- }
  PROCEDURE select_key( iIndex : BYTE; iPlayer : BYTE );
    VAR
      chInit : CHAR;
      chCur : CHAR; { , chPrev }
      bQuit : BOOLEAN;
      iFinal : BYTE;

      i : BYTE;

      bFound : BOOLEAN;

    BEGIN
      iFinal := ex2in( iIndex, iPlayer );
      chInit := g_Settings.Controls[ iFinal ];

      { Blink old key }
      TEXTCOLOR( LIGHTGRAY + BLINK );
      GOTOXY( 13, iIndex + 3 );
      WRITE( key_string( chInit ) );

      TEXTCOLOR( LIGHTGRAY );

      empty_buffer;

      bQuit := FALSE;
      REPEAT
        chCur := READKEY;

        CASE chCur OF
          KEY_NULL:
            BEGIN END; { IGNORE }

          KEY_RETURN, KEY_ESCAPE:
            BEGIN
              { Keep old key }
              GOTOXY( 13, iIndex + 3 );
              WRITE( key_string( chInit ) );
              bQuit := TRUE;

            END;

          '0'..'9', 'a'..'z', 'G', 'O', 'I', 'Q', 'R', 'S', 'K', 'M', 'H', 'P':
            BEGIN
              bFound := FALSE;
              i := 0;
              WHILE ( i <= MAX_CONTROLS ) AND NOT bFound DO
                BEGIN
                  IF ( i <> iFinal ) THEN
                    BEGIN
                      IF ( g_Settings.Controls[ i ] = chCur ) THEN
                        bFound := TRUE;

                    END;

                  IF NOT bFound THEN
                    INC( i );

                END;

              IF bFound THEN
                BEGIN
                  { Swap keys }
                  g_Settings.Controls[ i ] := g_Settings.Controls[ iFinal ];
                  g_Settings.Controls[ iFinal ] := chCur;

                  IF ( i DIV 4 + 1 = iPlayer ) THEN
                    BEGIN
                      GOTOXY( 13, in2ex( i ) + 3 );
                      WRITE( key_string( g_Settings.Controls[ i ] ) );

                    END;

                END
              ELSE
                BEGIN
                  { Set new key }
                  g_Settings.Controls[ iFinal ] := chCur;

                END;

              GOTOXY( 13, iIndex + 3 );
              WRITE( key_string( chCur ) );
              bQuit := TRUE;

            END;

        END;

      UNTIL bQuit;

    END;


{ --------------------------------------------------------------------------------- item_settings_controls_player ----- }
  PROCEDURE item_settings_controls_player( iSel : BYTE );
    VAR
      i : BYTE;

    BEGIN
      GOTOXY( 14, 1 );
      FOR i := 1 TO MAX_PLAYERS DO
        BEGIN
          IF ( i = iSel ) THEN
            TEXTCOLOR( LIGHTGRAY )
          ELSE
            TEXTCOLOR( DARKGRAY );

          WRITE( i, ' ' );

        END;
      TEXTCOLOR( LIGHTGRAY );

    END;


{ ---------------------------------------------------------------------------------------- menu_settings_controls ----- }
  PROCEDURE menu_settings_controls;
    CONST
      INDEX_SETTINGS_CONTROLS_PLAYER	= 0;
      INDEX_SETTINGS_CONTROLS_LEFT		= 1;
      INDEX_SETTINGS_CONTROLS_RIGHT		= 2;
      INDEX_SETTINGS_CONTROLS_UP		= 3;
      INDEX_SETTINGS_CONTROLS_DOWN		= 4;
      INDEX_SETTINGS_CONTROLS_DONE		= 5;

    VAR
      iPlayer : BYTE;

      iCur, iPrev : BYTE;
      bQuit : BOOLEAN;
      chKey : CHAR;
      iFrameLeft : BYTE;
      iY : ARRAY[ INDEX_SETTINGS_CONTROLS_PLAYER..INDEX_SETTINGS_CONTROLS_DONE ] OF BYTE;

      i : BYTE;

    BEGIN
      iY[ INDEX_SETTINGS_CONTROLS_PLAYER ]	:= 1;
      iY[ INDEX_SETTINGS_CONTROLS_LEFT ]	:= 3;
      iY[ INDEX_SETTINGS_CONTROLS_RIGHT ]	:= 4;
      iY[ INDEX_SETTINGS_CONTROLS_UP ]		:= 5;
      iY[ INDEX_SETTINGS_CONTROLS_DOWN ]	:= 6;
      iY[ INDEX_SETTINGS_CONTROLS_DONE ]	:= 8;

      iCur := INDEX_SETTINGS_CONTROLS_PLAYER;
      iPrev := iCur;
      iPlayer := 1;

      CLRSCR;
      iFrameLeft := frame_center( 24, 10, TRUE );
      WINDOW( iFrameLeft + 4, WORK_TOP + 1, iFrameLeft + 22, WORK_TOP + 9 );
      WRITELN( 'Player ' );
      WRITELN( '       ' );
      WRITELN( 'Left   ' );
      WRITELN( 'Right  ' );
      WRITELN( 'Up     ' );
      WRITELN( 'Down   ' );
      WRITELN( '       ' );
      WRITE  ( 'Done   ' );

      WINDOW( iFrameLeft + 1, WORK_TOP + 1, iFrameLeft + 22, WORK_TOP + 9 );

      { Items }
      item_settings_controls_player( iPlayer );
      FOR i := 0 TO 3 DO
        BEGIN
          GOTOXY( 13, i + 3 );
          WRITE( key_string( g_Settings.Controls[ ex2in( i, iPlayer ) ] ) );

        END;

      arrow_move( iY[ iPrev ], iY[ iCur ] );

      bQuit := FALSE;
      REPEAT
        chKey := READKEY;
        CASE chKey OF

          KEY_UP:
            BEGIN
              iPrev := iCur;
              iCur := ( iCur + 5 ) MOD 6;
              arrow_move( iY[ iPrev ], iY[ iCur ] );

            END;

          KEY_DOWN:
            BEGIN
              iPrev := iCur;
              iCur := ( iCur + 1 ) MOD 6;
              arrow_move( iY[ iPrev ], iY[ iCur ] );

            END;

          KEY_LEFT:
            BEGIN
              IF ( iCur <> INDEX_SETTINGS_CONTROLS_DONE ) THEN
                BEGIN
                  IF ( iPlayer > 1 ) THEN
                    BEGIN
                      DEC( iPlayer );
                      item_settings_controls_player( iPlayer );
                      FOR i := 0 TO 3 DO
                        BEGIN
                          GOTOXY( 13, i + 3 );
                          WRITE( key_string( g_Settings.Controls[ ex2in( i, iPlayer ) ] ) );

                        END;

                    END;

                END;

            END;

          KEY_RIGHT:
            BEGIN
              IF ( iCur <> INDEX_SETTINGS_CONTROLS_DONE ) THEN
                BEGIN
                  IF ( iPlayer < MAX_PLAYERS ) THEN
                    BEGIN
                      INC( iPlayer );
                      item_settings_controls_player( iPlayer );
                      FOR i := 0 TO 3 DO
                        BEGIN
                          GOTOXY( 13, i + 3 );
                          WRITE( key_string( g_Settings.Controls[ ex2in( i, iPlayer ) ] ) );

                        END;

                    END;

                END;

            END;

          KEY_RETURN, KEY_SPACE:
            BEGIN
              CASE iCur OF
                INDEX_SETTINGS_CONTROLS_DONE:
                  bQuit := TRUE;

                INDEX_SETTINGS_CONTROLS_LEFT..INDEX_SETTINGS_CONTROLS_DOWN:
                  select_key( iCur - 1, iPlayer );

              END;

            END;

          KEY_ESCAPE:
            BEGIN
              iCur := INDEX_SETTINGS_CONTROLS_DONE;
              bQuit := TRUE;

            END;

        END; { CASE }

      UNTIL bQuit;
      window_work;

    END;


{ --------------------------------------------------------------------------------------------------- menu_scores ----- }
  PROCEDURE menu_scores;
    VAR
      iFrameLeft : BYTE;
      iCurLen : BYTE;
      iMaxLen : BYTE;
      i : BYTE;

    BEGIN
      iMaxLen := 0;
      FOR i := 1 TO 5 DO
        BEGIN
          iCurLen := LENGTH( g_Scores[ i ].sName );
          IF ( iCurLen > iMaxLen ) THEN
            iMaxLen := iCurLen;

        END;

      iFrameLeft := frame_center( 18 + min( iMaxLen + 2, 20 ), 7, TRUE );
      WINDOW( iFrameLeft + 2, WORK_TOP + 1, iFrameLeft + 36, WORK_TOP + 6 );
      FOR i := 1 TO 5 DO
        BEGIN
          WRITELN( '0', i, '  ', g_Scores[i].iPoints:6, '  ', g_Scores[i].sName );

        END;

      any_key;
      window_work;

    END;


{ -------------------------------------------------------------------------------------------------- credit_movie ----- }
  PROCEDURE credit_movie( iFrame : BYTE );
    BEGIN
          GOTOXY( 1, 3 );
          CASE iFrame OF
            1:
              BEGIN
                WRITELN_CENTER( 'COPYRIGHTCILLAS ' + crc_VERSION );
                WRITELN_CENTER( 'Release ' + crc_RELEASE );
                WRITELN;
                WRITELN_CENTER( '"Forget about Nokia..."' );

              END;

            2:
              BEGIN
                WRITELN_CENTER( 'Designed and written by' );
                WRITELN;
                WRITELN_CENTER( 'Florian Oldach' );
                WRITELN_CENTER( 'Sebastian Pipping' );

              END;

            3:
              BEGIN
                WRITELN_CENTER( 'Some maps and good ideas' );
                WRITELN;
                WRITELN_CENTER( 'Christopher Garbe' );
                WRITELN_CENTER( 'Patricia Hhlemann' );
                WRITELN_CENTER( 'Katarina Schulz' );

              END;

            4:
              BEGIN
                WRITELN_CENTER( 'Thanks to our betatesters' );
                WRITELN;
                WRITELN_CENTER( 'Hanns Pipping' );
                WRITELN_CENTER( 'Christian Reiher' );
                WRITELN_CENTER( 'Jonas Wolf' );

              END;

            5:
              BEGIN
                WRITELN_CENTER( 'Contact and updates' );
                WRITELN;
                WRITELN_CENTER( 'www.hartwork.org' );
                WRITELN_CENTER( 'contact@hartwork.org' );

              END;

          END;

    END;


{ -------------------------------------------------------------------------------------------------- menu_credits ----- }
  PROCEDURE menu_credits;
    CONST
      NUMBER_OF_FRAMES = 5;
      DISPLAY_LENGTH   = 4000;
      FADE_SPEED       = 90;
      FRAME_PAUSE      = 500;

    VAR
      iFrame    : BYTE;
      iStep     : BYTE;
      iPart     : BYTE;
      iPartTime : WORD;

    BEGIN
      iFrame := 0;
      empty_buffer;
      WHILE TRUE DO
        BEGIN
          iFrame := iFrame MOD NUMBER_OF_FRAMES + 1; { loop 1..NUMBER_OF_FRAMES }
          FOR iStep := 1 TO 4 DO
            BEGIN
              CLRSCR;
              CASE iStep OF

                1:
                  BEGIN
                    { Pause between two frames }
                    iPartTime := FRAME_PAUSE DIV 10;
                    FOR iPart := 1 TO 10 DO
                      BEGIN
                        IF KEYPRESSED THEN EXIT;
                        DELAY( iPartTime );

                      END;

                  END;

                2, 4:
                  BEGIN
                    { Fade in/out }
                    credit_movie( iFrame );

                    iPartTime := FADE_SPEED DIV 10;
                    FOR iPart := 1 TO 10 DO
                      BEGIN
                        IF KEYPRESSED THEN EXIT;
                        DELAY( iPartTime );

                      END;

                  END;

                3:
                  BEGIN
                    { Display }
                    TEXTCOLOR( WHITE );
                    credit_movie( iFrame );
                    TEXTCOLOR( LIGHTGRAY );

                    iPartTime := DISPLAY_LENGTH DIV 10;
                    FOR iPart := 1 TO 10 DO
                      BEGIN
                        IF KEYPRESSED THEN EXIT;
                        DELAY( iPartTime );

                      END;

                  END;


              END;

            END;

        END;

    END;


{ ---------------------------------------------------------------------------------------------------------- menu ----- }
  FUNCTION menu( iPage : BYTE ) : BOOLEAN;
    VAR
      iIndexMain      : BYTE;
      iIndexSettings  : BYTE;
      bRunning        : BOOLEAN;

    BEGIN
      iIndexMain      := INDEX_MAIN_START;
      iIndexSettings  := INDEX_SETTINGS_GAME;
      bRunning := TRUE;

      CLRSCR;
      main_logo;
      main_info;
      window_work;

      WHILE bRunning DO
        BEGIN

          empty_buffer;
          CLRSCR;
          CASE iPage OF

            PAGE_MAIN:
              BEGIN
                iIndexMain  := menu_main( iIndexMain );
                iPage       := iIndexMain;

              END;

            PAGE_SETTINGS:
              BEGIN
                iIndexSettings := menu_settings( iIndexSettings );
                CASE iIndexSettings OF

                  INDEX_SETTINGS_GAME:
                    BEGIN
                      iPage := PAGE_SETTINGS_GAME;
                      iIndexSettings := INDEX_SETTINGS_BACK;

                    END;

                  INDEX_SETTINGS_MULTIPLAYER:
                    BEGIN
                      iPage := PAGE_SETTINGS_MULTIPLAYER;
                      iIndexSettings := INDEX_SETTINGS_BACK;

                    END;

                  INDEX_SETTINGS_CONTROLS:
                    BEGIN
                      iPage := PAGE_SETTINGS_CONTROLS;
                      iIndexSettings := INDEX_SETTINGS_BACK;

                    END

                ELSE
                  BEGIN
                    iIndexSettings  := INDEX_SETTINGS_GAME;
                    iIndexMain      := INDEX_MAIN_START;
                    iPage           := PAGE_MAIN;

                  END;


                END;

              END;

            PAGE_SETTINGS_GAME:
              BEGIN
                menu_settings_game;
                iPage := PAGE_SETTINGS;

              END;

            PAGE_SETTINGS_MULTIPLAYER:
              BEGIN
                menu_settings_multiplayer;
                iPage := PAGE_SETTINGS;

              END;

            PAGE_SETTINGS_CONTROLS:
              BEGIN
                menu_settings_controls;
                iPage := PAGE_SETTINGS;

              END;

            PAGE_SCORES:
              BEGIN
                menu_scores;
                iPage := PAGE_MAIN;

              END;

            PAGE_CREDITS:
              BEGIN
                menu_credits;
                iPage := PAGE_MAIN;

              END;

            PAGE_QUIT, PAGE_START:
              bRunning := FALSE;

          END; { CASE }

        END; { WHILE }

      window_full_25;
      IF ( iPage = PAGE_START ) THEN
        menu := TRUE
      ELSE
        menu := FALSE;

    END;


{ ---------------------------------------------------------------------------------------------------------- INIT ----- }
BEGIN
  RANDOMIZE;

END.