{

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
    *                                                                 *
    *   Copyrightcillas 1.0.3                                         *
    *   Copyright  2004  Florian Oldach <oldach@hartwork.org>        *
    *                     Sebastian Pipping <pipping@hartwork.org>    *
    *                                                                 *
    *   --> http://www.hartwork.org                                   *
    *                                                                 *
    *                                                                 *
    *   This source code is released under LGPL.                      *
    *   See LGPL.txt for details.                        2004-09-01   *
    *                                                                 *
    \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

}

UNIT crc_std;

{ ----------------------------------------------------------------------------------------------------- INTERFACE ----- }
INTERFACE

  USES
    crt,
    dos;


  CONST
    CRC_VERSION         = '1.0.3';
    CRC_RELEASE         = '2004-09-01';

    MAX_PLAYERS         = 4;
    MAX_CONTROLS        = MAX_PLAYERS * 4 - 1;
    MAX_ROUTES          = MAX_PLAYERS * ( MAX_PLAYERS + 1 ) DIV 2;

    MAX_CORNERS         = 7;
    DEFAULT_GROW        = 10;


    MAX_NAME_LENGTH     = 20;

    FILE_MAPS           = 'maps.dat';
    FILE_SCORES         = 'scores.dat';
    FILE_SETTINGS       = 'settings.dat';

    KEY_RETURN          = #13;
    KEY_ESCAPE          = #27;
    KEY_SPACE           = #32;
    KEY_BACKSPACE       = #8;

    KEY_NULL            = #0;

    { With leading zero: }
    KEY_LEFT            = #75;
    KEY_RIGHT           = #77;
    KEY_UP              = #72;
    KEY_DOWN            = #80;

    KEY_INSERT          = 'R';
    KEY_DELETE          = 'S';
    KEY_HOME            = 'G';
    KEY_END             = 'O';
    KEY_PAGEUP          = 'I';
    KEY_PAGEDOWN        = 'Q';

    SCREEN_WIDTH        = 80;
    SCREEN_HEIGHT       = 25;

    ASCII_NULL          = #0;
    ASCII_SPACE         = #32;
    ASCII_UNDERLINE     = #95;
    ASCII_LOWER         = #60;
    ASCII_GREATER       = #62;
    ASCII_SHIFT_LEFT    = #174;
    ASCII_SHIFT_RIGHT   = #175;

    ASCII_DOT           = #250;
    ASCII_BLOCK_MID     = #254;
    ASCII_CIRCLE        = #9;

    ASCII_ARROW_UP      = #30;
    ASCII_ARROW_RIGHT   = #16;
    ASCII_ARROW_DOWN    = #31;
    ASCII_ARROW_LEFT    = #17;
    ASCII_CORNER_NE     = #192;
    ASCII_CORNER_ES     = #218;
    ASCII_CORNER_SW     = #191;
    ASCII_CORNER_WN     = #217;
    ASCII_LINE_HORZ     = #196;
    ASCII_LINE_VERT     = #179;

    ASCII_DOUBLE_CORNER_NE     = #200;
    ASCII_DOUBLE_CORNER_ES     = #201;
    ASCII_DOUBLE_CORNER_SW     = #187;
    ASCII_DOUBLE_CORNER_WN     = #188;
    ASCII_DOUBLE_LINE_HORZ     = #205;
    ASCII_DOUBLE_LINE_VERT     = #186;


    ASCII_SNAKE         = 'S';
    ASCII_COOKIE        = ''; {#184}

    MAP_WIDTH           = 80;
    MAP_HEIGHT          = 49;

    DIR_UP              = 0;
    DIR_RIGHT           = 1;
    DIR_DOWN            = 2;
    DIR_LEFT            = 3;


  TYPE
    tName = STRING[ MAX_NAME_LENGTH ];

    tBytePoint = RECORD
      x : BYTE;
      y : BYTE;

    END;

    tRelative = RECORD
      iDir  : BYTE;
      iDist : BYTE;

    END;

    tSnakeBirth = RECORD
      Start    : tBytePoint;
      Path     : ARRAY[ 1..MAX_CORNERS ] OF tRelative;
      iCorners : BYTE;
      iDirHead : BYTE;
      iDirFoot : BYTE;

    END;

    tMap = ARRAY [ 1..MAP_WIDTH, 1..MAP_HEIGHT ] OF CHAR;

    tMapEx = RECORD
      sTitle           : STRING[ MAX_NAME_LENGTH ];
      iInitialCookies  : BYTE;
      bRefreshCookies  : BOOLEAN;
      chMap            : tMap;
      Routes           : ARRAY[ 1..MAX_ROUTES ] OF tSnakeBirth;

    END;

    tFileMap = FILE OF tMapEx;

  PROCEDURE writeln_center( szText : STRING );
  FUNCTION  frame_center( iWidth, iHeight : BYTE; bAlignTop : BOOLEAN ) : BYTE;
  PROCEDURE window_full_25;
  PROCEDURE window_full_50;
  PROCEDURE window_center( iWidth, iHeight, iOffsetX, iOffsetY : BYTE );
  PROCEDURE empty_buffer;
  PROCEDURE any_key;
  PROCEDURE special_key;
  FUNCTION  continue : BOOLEAN;
  PROCEDURE enable_cursor( on : BOOLEAN );
  PROCEDURE res_def;
  PROCEDURE res_small;
  PROCEDURE res_big;
  FUNCTION  get_window_width : BYTE;
  FUNCTION  zero( iValue : WORD; iNOZ : BYTE ) : STRING;
  PROCEDURE clear_map( VAR MyMap : tMap );
  FUNCTION  between( iNumber, iMin, iMax : BYTE ) : BOOLEAN;
  FUNCTION  member( x, y : BYTE ) : BOOLEAN;
  FUNCTION  min( a, b : BYTE ) : BYTE;


{ ------------------------------------------------------------------------------------------------ IMPLEMENTATION ----- }
IMPLEMENTATION

  VAR
    iOrgMode : BYTE;


{ ------------------------------------------------------------------------------------------------ writeln_center ----- }
  PROCEDURE writeln_center( szText : STRING );
    VAR
      iLen : BYTE;

    BEGIN
      iLen := LENGTH( szText );
      GOTOXY( ( get_window_width - iLen ) DIV 2 + 1, WHEREY );
      WRITELN( szText );

    END;


{ -------------------------------------------------------------------------------------------------- frame_center ----- }
  FUNCTION frame_center( iWidth, iHeight : BYTE; bAlignTop : BOOLEAN ) : BYTE;
    VAR
      iLeft  : BYTE;
      iTop   : BYTE;
      i      : BYTE;

    BEGIN
      iLeft := ( 80 - iWidth ) DIV 2 + 1;
      IF bAlignTop THEN
        iTop := 1
      ELSE
        iTop := ( 50 - iHeight ) DIV 2 + 1;

      GOTOXY( iLeft, iTop );
      WRITE( ASCII_CORNER_ES );
      FOR i := 2 TO iWidth - 1 DO
        WRITE( ASCII_LINE_HORZ );
      WRITE( ASCII_CORNER_SW );

      FOR i := iTop + 1 TO iTop + iHeight - 2 DO
        BEGIN
          GOTOXY( iLeft, i );
          WRITE( ASCII_LINE_VERT );
          GOTOXY( iLeft + iWidth - 1, i );
          WRITE( ASCII_LINE_VERT );

        END;

      GOTOXY( iLeft, iTop + iHeight - 1 );
      WRITE( ASCII_CORNER_NE );
      FOR i := 2 TO iWidth - 1 DO
        WRITE( ASCII_LINE_HORZ );
      WRITE( ASCII_CORNER_WN );

      frame_center := iLeft;

    END;


{----------------------------------------------------------------------------------------------------window_full_25-----}
  PROCEDURE window_full_25;
    BEGIN
      WINDOW( 1, 1, 80, 25 );

    END;


{----------------------------------------------------------------------------------------------------window_full_50-----}
  PROCEDURE window_full_50;
    BEGIN
      WINDOW( 1, 1, 80, 50 );

    END;


{ ------------------------------------------------------------------------------------------------- window_center ----- }
  PROCEDURE window_center( iWidth, iHeight, iOffsetX, iOffsetY : BYTE );
    VAR
      ax, ay, bx, by : BYTE;

    BEGIN
      ax := ( 80 - iWidth ) DIV 2 + 1 + iOffsetX;
      ay := ( 50 - iHeight ) DIV 2 + 1 + iOffsetY;
      bx := ax + iWidth - 1 + iOffsetX;
      by := ay + iHeight - 1 + iOffsetY;
      WINDOW( ax, ay, bx, by );

    END;


{ --------------------------------------------------------------------------------------------------- emty_buffer ----- }
  PROCEDURE empty_buffer;
    VAR
      chKey : CHAR;

    BEGIN
      WHILE KEYPRESSED DO
        chKey := READKEY;
    END;


{ ------------------------------------------------------------------------------------------------------- any_key ----- }
  PROCEDURE any_key;
    VAR
      chKey : CHAR;

    BEGIN
      empty_buffer;
      chKey := READKEY;

    END;


{ --------------------------------------------------------------------------------------------------- special_key ----- }
  PROCEDURE special_key; { [RETURN], [ESCAPE], [SPACE] }
    VAR
      chKey   : CHAR;
      bQuit   : BOOLEAN;

    BEGIN
      bQuit   := FALSE;

      empty_buffer;
      REPEAT
        chKey := READKEY;

        CASE chKey OF

          KEY_RETURN, KEY_ESCAPE, KEY_SPACE:
            bQuit := TRUE;

        END;

      UNTIL bQuit;

    END;


{ ------------------------------------------------------------------------------------------------------ continue ----- }
  FUNCTION continue: BOOLEAN; { [RETURN], [ESCAPE], [SPACE] }
    VAR
      chKey : CHAR;
      bDone : BOOLEAN;

    BEGIN
      bDone := FALSE;
      empty_buffer;
      REPEAT
        chKey := READKEY;

        CASE chKey OF

          KEY_RETURN, KEY_SPACE:
            BEGIN
              continue := TRUE;
              bDone := TRUE;

            END;

          KEY_ESCAPE:
            BEGIN
              continue := FALSE;
              bDone := TRUE;

            END;

        END;

      UNTIL bDone;

    END;


{ ------------------------------------------------------------------------------------------------- enable_cursor ----- }
  PROCEDURE enable_cursor( on : BOOLEAN );
    VAR
      regs : REGISTERS ;

    BEGIN
      regs.ax := $0100 ;
      IF on THEN
        regs.cx := $0607
      ELSE
        regs.cx := $2607;
        INTR( $10, regs );

    END;


{ ------------------------------------------------------------------------------------------------------- res_def ----- }
  PROCEDURE res_def;
    BEGIN
      TEXTMODE( iOrgMode );
      enable_cursor( TRUE );
      TEXTCOLOR( LIGHTGRAY );
      CLRSCR;

    END;


{ ----------------------------------------------------------------------------------------------------- res_small ----- }
  PROCEDURE res_small;
    BEGIN
      TEXTMODE( CO80 );
      enable_cursor( FALSE );

    END;


{ ------------------------------------------------------------------------------------------------------- res_big ----- }
  PROCEDURE res_big;
    BEGIN
      TEXTMODE( CO80 + Font8x8 );
      enable_cursor( FALSE );

    END;


{ ---------------------------------------------------------------------------------------------- get_window_width ----- }
  FUNCTION get_window_width : BYTE;
    BEGIN
      get_window_width := LO( WINDMAX ) - LO( WINDMIN ) + 1;

    END;


{ ---------------------------------------------------------------------------------------------------------- zero ----- }
  FUNCTION zero( iValue : WORD; iNOZ : BYTE ) : STRING;
    VAR
      t1, t2 : STRING;
      iDif : INTEGER;
      i : BYTE;

    BEGIN
      STR( iValue, t1 );
      iDif := iNOZ - LENGTH( t1 );
      IF ( iDif > 0 ) THEN
        BEGIN
          t2 := '';
          FOR i := 1 TO iDif DO
            t2 := t2 + '0';
          zero := t2 + t1;
        END
      ELSE
        zero := t1;

    END;


{ ----------------------------------------------------------------------------------------------------- clear_map ----- }
  PROCEDURE clear_map( VAR MyMap : tMap );
    VAR
      x, y : BYTE;

    BEGIN
      FOR y := 1 TO MAP_HEIGHT DO
        FOR x := 1 TO MAP_WIDTH DO
          MyMap[ x, y ] := ' ';

    END;


{ ------------------------------------------------------------------------------------------------------- between ----- }
  FUNCTION between( iNumber, iMin, iMax : BYTE ) : BOOLEAN;
    BEGIN
      IF ( iNumber < iMin ) OR ( iNumber > iMax ) THEN
        between := FALSE

      ELSE
        between := TRUE;

    END;


{ -------------------------------------------------------------------------------------------------------- member ----- }
  FUNCTION member( x, y : BYTE ) : BOOLEAN;
    BEGIN
      IF between( x, 1, MAP_WIDTH ) AND between( y, 1, MAP_HEIGHT ) THEN
        member := TRUE

      ELSE
        member := FALSE;

    END;


{ ----------------------------------------------------------------------------------------------------------- min ----- }
  FUNCTION min( a, b : BYTE ) : BYTE;
    BEGIN
      IF ( a < b ) THEN
        min := a
      ELSE
        min := b;

    END;


{ ---------------------------------------------------------------------------------------------------------- INIT ----- }
BEGIN
  iOrgMode := LASTMODE;

END.